from pygame.locals import * # for event MOUSE variables
from constants import * # for game design variables
from songsList import *
from alib import * # accelerometer library
import pygame, os, time, random, math, movement

if piTFT:
    os.putenv('SDL_VIDEODRIVER', 'fbcon') # Display on piTFT
    os.putenv('SDL_FBDEV', '/dev/fb0') #1 for piTFT w monitor
    os.putenv('SDL_MOUSEDRV', 'TSLIB') # Track mouse clicks on piTFT
    os.putenv('SDL_MOUSEDEV', '/dev/input/touchscreen')

pygame.init()

pygame.mouse.set_visible(True) # False for PITFT

t_font = pygame.font.Font(None, 30) # Text font
current_song_number = 0
# Location of text on home screen. Pos defined in constant
home_text = {title_pos: 'Music Motion', start_pos: 'Start', quit_pos: 'Quit',
             select_song_pos: 'Song Selected', prev_pos: '<<',
             next_pos: '>>', curr_song_pos: songsList[current_song_number].title}
# Location of text on play screen.
play_text = {left_pos: "Left", right_pos: "Right", start_pos: 'Home'}
# Location of text on score screen. Reuse positions from Home text
score_text = {title_pos: "Total Score", select_song_pos: "", start_pos: 'Home'}

run = True
home = True # Whether or not currently on the home screen.
scoreScreen = False # Whether or not to show the score screen
song = None
prev_time = 0


"""
Blits text at pos on screen. Text color defaults to t_color
"""
def blitText(text, pos):
    text_surface = t_font.render(text, True, t_color)
    rect = text_surface.get_rect(center=pos)
    screen.blit(text_surface, rect)

"""
Updates home screen.
Erases current screen and draws all text in home_text dictionary.
"""
def drawHome():
    screen.fill(b_color) # Erase the workspace
    for pos, text in home_text.items():
        blitText(text, pos)
    pygame.display.flip()

"""
Updates score screen.
Erases current screen and draws all text in score_text dictionary.
"""
def drawScore(score):
    score_text[select_song_pos] = str(score)
    screen.fill(b_color) # Erase the workspace
    for pos, text in score_text.items():
        blitText(text, pos)
    pygame.display.flip()

"""
Draws a single arrow onto the screen. lcolor is color of arrow and start and
end represent positions of start and end of arrow. Trirad is the size
of the arrowhead.

See: https://stackoverflow.com/questions/43527894/drawing-arrowheads-which-follow-the-direction-of-the-line-in-pygame/43529178
"""
def drawArrow(screen, lcolor, start, end, trirad = 20):
    pygame.draw.line(screen,lcolor,start,end,2)
    rotation = math.degrees(math.atan2(start[1]-end[1], end[0]-start[0]))+90
    pygame.draw.polygon(screen, lcolor, ((end[0]+trirad*math.sin(math.radians(rotation)), end[1]+trirad*math.cos(math.radians(rotation))), (end[0]+trirad*math.sin(math.radians(rotation-120)), end[1]+trirad*math.cos(math.radians(rotation-120))), (end[0]+trirad*math.sin(math.radians(rotation+120)), end[1]+trirad*math.cos(math.radians(rotation+120)))))

"""
Draws left and right arrows. Use constants to place all 8 arrows in the
correct positions. move is of type Movement and holds information on which
arrows are the current "action" arrows.
"""
def drawArrows(screen, move):
    # Draw left arrows
    drawArrow(screen, move.leftColor(0), l_up_pos[0], l_up_pos[1])
    drawArrow(screen, move.leftColor(1), l_left_pos[0], l_left_pos[1])
    drawArrow(screen, move.leftColor(2), l_right_pos[0], l_right_pos[1])
    drawArrow(screen, move.leftColor(3), l_down_pos[0], l_down_pos[1])

    # Draw right arrows
    drawArrow(screen, move.rightColor(0), r_up_pos[0], r_up_pos[1])
    drawArrow(screen, move.rightColor(1), r_left_pos[0], r_left_pos[1])
    drawArrow(screen, move.rightColor(2), r_right_pos[0], r_right_pos[1])
    drawArrow(screen, move.rightColor(3), r_down_pos[0], r_down_pos[1])

"""
Draws a single queue arrow centered at (x, y) facing the direction (one of 0
(UP), 1 (LEFT), 2 (RIGHT), or 3 (DOWN))
"""
def drawQueueArrow(screen, isRight, x, y, dir):
    delta = queue_arrow_length // 2
    color = right_color if isRight else left_color
    start = [x,y]
    end = [x,y]
    if dir == -1: # No direction
        return
    elif dir == 0:
        start[1] += delta
        end[1] -= delta
    elif dir == 1:
        start[0] += delta
        end[0] -= delta
    elif dir == 2:
        start[0] -= delta
        end[0] += delta
    elif dir == 3:
        start[1] -= delta
        end[1] += delta
    else: # Invalid direction
        raise Exception('Direction for queue arrow should be in one of -1, 0, 1, 2, 3')
    drawArrow(screen, color, start, end, 8)

"""
Draws a single queue with first and second positions (each one of 0 (UP), 1
(LEFT), 2 (RIGHT), or 3 (DOWN)) and depending on whether it is the right or
left queue, positions the x of the queue arrows accordinly.
"""
def drawQueue(screen, isRight, first, second):
    x_center = 3*eighth_w
    y_center = 3*quarter_h # y_center of first row
    if isRight:
        x_center += quarter_w
    drawQueueArrow(screen, isRight, x_center, y_center, first)
    drawQueueArrow(screen, isRight, x_center, y_center + quarter_h // 2, second)

"""
Draws both queues based on the two upcoming moves, move1 and move2.
"""
def drawQueues(screen, move1, move2):
    drawQueue(screen, False,  move1.getLeft(), move2.getLeft())
    drawQueue(screen, True,  move1.getRight(), move2.getRight())

"""
Updates game screen during play.
Erases current screen and draws the game screen with correct current move
(correct arrows highlighted) and move queue.
"""
def drawPlay(screen, song):
    screen.fill(b_color)
    for pos, text in play_text.items():
        blitText(text, pos)
    drawArrows(screen, song.currMove())
    drawQueues(screen, song.nextMove(), song.nextNextMove())
    pygame.display.flip()

# Initialize screen with home text
screen = pygame.display.set_mode((width, height))
drawHome()

score = 0
total = 0

while (run):
    # get data for left hand = s0
    xyz0 = get_aves(s0)
    x0 = xyz0[0]
    z0 = xyz0[2]
    # get data for right hand = s1
    xyz1 = get_aves(s1)
    x1 = xyz1[0]
    z1 = xyz1[2]

    # Either the play or score screen.
    if not(home):

        # Song is playing. Draw correct move on screen.
        if time.time() - prev_time > (1 / song.bpm)*60 and not(scoreScreen):
            #print(score)
            prev_time = time.time()
            if song.incrementMove():
                drawPlay(screen, song)
                if lmove != -1:
                    total = total + 2
            else:
                pygame.mixer.music.stop()
                drawScore(score)
                scoreScreen = True

        # get accelerometer data and score

        # get current move
        curr = song.currMove()
        lmove = curr.getLeft()
        rmove = curr.getRight()

        # determine move for sensor0
        if ready_move(s0): # if delay period is over
            move = get_lmove(x0, z0)
            if move!=-1: # a move has occurred
                if rmove == move:
                    pause_time0 = time.time() # set start of delay
                    #print_move(move, "L")
                    score = score + 1

        # determine move for sensor1
        if ready_move(s1):
            move = get_rmove(x1, z1)
            if move!=-1: # a move has occurred
                if lmove == move:
                    pause_time1 = time.time() # set start of delay
                    #print_move(move, "R")
                    score = score + 1

    for event in pygame.event.get():
        if(event.type == MOUSEBUTTONUP):
            pos = pygame.mouse.get_pos()
            x,y = pos
            song = songsList[current_song_number]

            # Click in the lower part of the screen
            if y > 3*quarter_h:
                if x > 3*quarter_w and home:# Quit
                        run = False
                if x < quarter_w:
                    scoreScreen = False
                    if home: # Start
                        score = 0
                        home = False
                        song.reset()
                        prev_time = time.time()
                        pygame.mixer.music.load(song.fileTitle)
                        # Start song at offset seconds
                        pygame.mixer.music.play(start=song.offset)
                        drawPlay(screen, song)
                    else: # Home
                        pygame.mixer.music.stop()
                        home = True
                        drawHome()

            # Click one of the << or >> song selction arrows
            if y > half_h and y < 3*quarter_h and home:
                if x > 3*quarter_w: # Next song
                    current_song_number += 1
                if x < quarter_w: # Prev song
                    current_song_number -= 1
                current_song_number = current_song_number % len(songsList)
                song = songsList[current_song_number]
                home_text[curr_song_pos] = song.title
                drawHome() # Update screen with newly selected song
